﻿init python:
    import pygame
    from copy import copy
    import random
    import time
    random.seed(time.clock())
    def distance(a, b):
        if a >= b:
            result = a - b
        else:
            result = b - a
        return result
    renpy.music.register_channel("sfx1", "sfx", True)
    class cars_lane_class(renpy.Displayable):
        def __init__(self, image):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.player = None
            self.x = 0
            self.y = 0
        def render(self,width,height,st,at):
            img = self.image
            t = Transform(img, xanchor=0.5, yanchor=0.5, rotate = 0)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x,self.y))

            if self.y < 150:
                renpy.redraw(self, 0)
            return rv
    class cars_c_class(renpy.Displayable):
        def __init__(self, image, speed):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.player = None
            self.x = 0
            self.y = -150
        def render(self,width,height,st,at):
            img = self.image
            t = Transform(img, xanchor=0.5, yanchor=0.5, rotate = 0)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x,self.y))

            if self.y < 150:
                renpy.redraw(self, 0)
            return rv

    class cars_s_class(renpy.Displayable):
        def __init__(self, image, lanes, cars, edge_distance):
            super(renpy.Displayable,self).__init__()
            self.image = image
            self.animation = 1

            self.lanes = []
            self.edge_distance = edge_distance
            for i in lanes:
                lane = copy(i[0])
                lane.x = i[1]
                self.lanes.append(lane)

            self.cars = cars
            self.active_cars = []

            self.cars_passed = 0

            self.speed = 0

            self.x = 0
            self.temp_x = 0
            self.y = 96
        def render(self,width,height,st,at):
            if self.speed:
                if self.x > self.temp_x:
                    self.x -= self.speed
                    if self.x < self.temp_x:
                        self.x = self.temp_x
                elif self.x < self.temp_x:
                    self.x += self.speed
                    if self.x > self.temp_x:
                        self.x = self.temp_x
                    
                if abs(self.x) > self.edge_distance:
                    self.speed = 0
                    Play("sound", "cars/crash.ogg")()
                    Stop("sfx1")()
            
                for i in self.lanes:
                    i.y += self.speed+1
                    if i.y >= 32:
                        i.y -= 32

                for i in self.active_cars:
                    i.y += self.speed
                    if i.y > 150:
                        self.active_cars.remove(i)
                        self.cars_passed += 1
                    if 70 < i.y < 120:
                        if distance(i.x, self.x) < 12:
                            self.speed = 0
                            Play("sound", "cars/crash.ogg")()
                            Stop("sfx1")()
            img = "{} {}".format(self.image, self.animation)
            t = Transform(img, xanchor=0.5, yanchor=0.5, rotate = 0)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x,self.y))

            renpy.redraw(self, 0)
            return rv
        def event(self, ev, x, y, st):
            if ev.type == pygame.KEYDOWN:
                if ev.key == pygame.K_a:
                    if self.speed:
                        self.temp_x -= 32
                        renpy.play("cars/swerve.ogg", "sound")
                    raise renpy.IgnoreEvent()

                elif ev.key == pygame.K_d:
                    if self.speed:
                        self.temp_x += 32
                        renpy.play("cars/swerve.ogg", "sound")
                    raise renpy.IgnoreEvent()

                if ev.key == pygame.K_w:
                    if self.speed:
                        if self.speed < 4:
                            self.speed += 1
                            rev = "cars/rev {}.ogg".format(self.speed)
                            Play("sfx1", rev, loop = True)()
                    raise renpy.IgnoreEvent()

                elif ev.key == pygame.K_s:
                    if self.speed:
                        if self.speed > 1:
                            self.speed -= 1
                            rev = "cars/rev {}.ogg".format(self.speed)
                            Play("sfx1", rev, loop = True)()
                    raise renpy.IgnoreEvent()
                
            return None

        def spawn(self):
            car = copy(random.choice(self.cars))
            car.player = self
            car.x = random.randint(-1,1)*32
            self.active_cars.append(car)
        def animate(self):
            self.animation += 1
            if self.animation > 3:
                self.animation = 1
        def restart(self):
            self.active_cars = []
            self.speed = 2
            self.x = 0
            self.temp_x = 0
            self.cars_passed = 0
            Play("sfx1", "cars/rev 2.ogg", loop = True)()

image your_car 1 = Crop((0, 0,18, 30), "cars/your_car.png")
image your_car 2 = Crop((18, 0,18, 30), "cars/your_car.png")
image your_car 3 = Crop((36, 0,18, 30), "cars/your_car.png")


image cars_1 = Crop((0, 0,16, 32), "cars/cars.png")
image cars_2 = Crop((16, 0,16, 32), "cars/cars.png")
image cars_3 = Crop((32, 0,16, 32), "cars/cars.png")
image cars_4 = Crop((48, 0,16, 32), "cars/cars.png")
image cars_5 = Crop((64, 0,16, 32), "cars/cars.png")

image cars_6 = Crop((0, 32,18, 32), "cars/cars.png")
image cars_7 = Crop((20, 32,20, 32), "cars/cars.png")
image cars_8 = Crop((40, 32,20, 36), "cars/cars.png")
image cars_9 = Crop((62, 32,18, 32), "cars/cars.png")

default cars_lemon = cars_c_class("cars_1", 6)
default cars_taxi = cars_c_class("cars_2", 10)
default cars_red = cars_c_class("cars_3", 14)
default cars_silver = cars_c_class("cars_4", 16)
default cars_mini = cars_c_class("cars_5", 5)
# default cars_convert = cars_c_class("cars_6", 14)
default cars_police = cars_c_class("cars_7", 14)
default cars_van = cars_c_class("cars_8", 12)
default cars_yellow = cars_c_class("cars_9", 18)


default cars_left_lane = cars_lane_class("cars/left.png")
default cars_mid_lane = cars_lane_class("cars/lane.png")
default cars_right_lane = cars_lane_class("cars/right.png")

default test_my_car = cars_s_class(
    "your_car",
    [
        [cars_left_lane, -52],
        [cars_mid_lane, -32],
        [cars_mid_lane, 0],
        [cars_mid_lane, 32],
        [cars_right_lane, 52],
    ],
    [
        cars_lemon,
        cars_taxi,
        cars_red,
        cars_silver,
        cars_mini,
        # cars_convert,
        cars_police,
        cars_van,
        cars_yellow,
    ],
    44,
    )



transform zoom(z):
    nearest True
    zoom z

screen cars_screen(g = test_my_car):
    modal True
    style_prefix "cars"
    if g.speed:
        timer random.random()+(.7 -(g.speed*.1)) repeat True action Function(g.spawn)
        timer .1 repeat True action Function(g.animate)
    fixed xysize 480,270:
        at zoom(4)
        add "#4ec114"

        for i in g.lanes:
            add i align .5,1.0

        add g align .5,.5
        for i in g.active_cars:
            add i align .5,.5
    vbox align .0,.0 spacing 10 offset 10,10:
        text "Cars passed: [g.cars_passed]"
        text "Gear: [g.speed]"

    vbox align .0,1.0 spacing 10 offset 10,-10:
        # text str(len(g.active_cars))
        button:
            text "Skip"
            action Return()
        button:
            text "Restart"
            action Function(g.restart)

style cars_button:
    background "#0009"
    padding (20,20)
style cars_button:
    background "#0009"
    padding (20,20)


label cars_example:
    call screen cars_screen
